# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from pyrogram.types import InlineKeyboardButton


START = """
Hey {user}

Welcome to {bot}

I can {1}
Use below buttons to learn more.

By @StarkBots
"""

HELP = """
{1}

✨ **Available Commands** ✨

{commands}
"""

ABOUT = """
**About This Bot** 

A telegram bot to {1}{2}

Library : [PyStark](https://github.com/StarkBotsIndustries/pystark)

Language : [Python](www.python.org)

Developer : @StarkProgrammer
"""

HOME_BUTTON = [
    [InlineKeyboardButton(text="🏠 Return Home 🏠", callback_data="home")]
]

MAIN_BUTTONS = [
    [InlineKeyboardButton("✨ Bot Status and More Bots ✨", url="https://t.me/StarkBots/7")],
    [
        InlineKeyboardButton("How to Use ❔", callback_data="help"),
        InlineKeyboardButton("🎪 About 🎪", callback_data="about")
    ],
    [InlineKeyboardButton("♥ More Amazing bots ♥", url="https://t.me/StarkBots")],
]



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from pystark import Stark
from pystark.logger import logger
from pyrogram.errors import PeerIdInvalid
from pystark.database.sql import Database
from pystark.env import OWNER_ID, settings

db = Database()
module = settings


async def replace(m, msg, bot):
    # I do know str.format() but that'll be unnecessary execution of multiple things and anyway these are optional.
    # For same reason, IF statements are used below. Replies should be fast no matter what.
    # Also, prevents
    if '{user}' in m:
        m = m.replace("{user}", msg.from_user.first_name)
    if '{bot}' in m:
        m = m.replace("{bot}", (await bot.get_me()).first_name)
    if '{user_mention}' in m:
        m = m.replace("{user_mention}", msg.from_user.mention)
    if '{bot_mention}' in m:
        m = m.replace("{bot_mention}", (await bot.get_me()).mention)
    if '{owner}' in m:
        owner = '@pystark'
        if OWNER_ID:
            try:
                owner = (await bot.get_users(OWNER_ID[0])).mention
            except PeerIdInvalid:
                logger.warn(f"Can't interact with bot owner [user={OWNER_ID[0]}]. Please send a message to bot.")
        m = m.replace("{owner}", owner)
    return m

# ToDO
#   1. Don't repeat replace() function everywhere.


async def send_buttons():
    return False


LOADED_BUT_EMPTY = "You loaded '{}' addon but {} message is empty"

cache_commands = ""


async def replace_commands(bot: Stark, text: str):
    global cache_commands
    if cache_commands:
        return cache_commands
    basics = {"1": "", "2": "", "3": "", "4": ""}
    others = []
    cmds = bot.all_commands
    for c in cmds:
        if cmds[c]:
            x = f"/{c} - {cmds[c]} \n"
            if c == "start":
                basics["1"] = x
            elif c == "help":
                basics["2"] = x
            elif c == "about":
                basics["3"] = x
            elif c == "id":
                basics["4"] = x
            else:
                others.append(x)
    basics_str = basics["1"] + basics["2"] + basics["3"] + basics["4"]
    others_str = "".join(others)
    text = text.replace("{commands}", others_str+basics_str)
    cache_commands = text
    return text



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


import os
import pytz
import logging
import datetime
from pytz import UnknownTimeZoneError


class Formatter(logging.Formatter):
    """ Override logging.Formatter"""
    # Taken from https://stackoverflow.com/a/67241679

    def formatTime(self, record, datefmt):
        date = datetime.datetime.fromtimestamp(record.created, tz=pytz.UTC)
        try:
            timezone = os.environ.get("TIMEZONE")
            if not timezone:
                timezone = "Asia/Kolkata"
            date = date.astimezone(pytz.timezone(timezone))
        except UnknownTimeZoneError:
            print("WARNING - The timezone you filled doesn't exist. Please correct it. Till then, Indian timezone (Asia/Kolkata) will be used.")
            date = date.astimezone(pytz.timezone("Asia/Kolkata"))
        soln = date.strftime(datefmt)
        return soln


logger = logging.getLogger("stark_log")
logger.setLevel(logging.INFO)
console = logging.StreamHandler()
console.setLevel(logging.INFO)
formatter = Formatter('%(asctime)s - %(levelname)s - %(message)s', '%Y-%m-%d %H:%M:%S')
console.setFormatter(formatter)
logger.addHandler(console)
logger.propagate = False



from pystark import env
from pystark.utils import patch
from pyrogram.types import User
# from pystark.others.database import get_user


@patch(User)
class User:
    def is_sudo(self):
        # return (self.id in env.OWNER_ID) or (self.id in env.SUDO_USERS)
        return self.id in env.SUDO_USERS

    def is_owner(self):
        return self.id in env.OWNER_ID

    # async def get_lang(self):
    #     user = await get_user(self.id)
    #     if user.get("lang"):
    #         return user["lang"]
    #     else:
    #         return "en"



# Default Configuration for your project

DIRECTORY = "stark"

PLUGINS = f"{DIRECTORY}/plugins"

BOT_PLUGINS = f"{DIRECTORY}/assistant"

LOCALIZATION = f"{DIRECTORY}/localization"

SET_BOT_MENU = True

CMD_PREFIXES = ["."]

BOT_CMD_PREFIXES = ["/"]

START = "Thank you for starting this bot."

HELP = """
Available Commands are:

{commands}
"""

ABOUT = "About This Bot \nDeveloped using @PyStark"

TIMEZONE = "Asia/Kolkata"

START_IN_GROUPS = "Hey :) Ping me privately if you have any questions on how to use me."



import os
from sqlalchemy import inspect
from pystark.utils import patch
from pystark.logger import logger
from sqlalchemy.engine.base import Connection
from sqlalchemy import create_engine, MetaData


@patch(Connection)
class Connection:
    async def get(self, table_name: str, pk_column: str, primary_key) -> dict | None:
        x = self.execute("SELECT * FROM %s WHERE %s='%s'" % (table_name, pk_column, primary_key))
        x = [getattr(i, "_asdict")() for i in x]
        if x:
            return x[0]
        else:
            return None

    async def set(self, table_name: str, pk_column: str, primary_key, data: dict):
        text = ""
        for i in data:
            if isinstance(data[i], str):
                text += f"""{i}='{data[i].replace("'", "''")}'"""
            else:
                text += f"{i}={data[i]}"
            text += ", "
        data = text[:-2]
        self.execute(f"""UPDATE {table_name} SET {data} WHERE {pk_column}={primary_key}""")

    async def add(self, table_name: str, pk_column: str, primary_key, data: dict = None):
        if not data:
            data = {}
        data[pk_column] = primary_key
        data = {i: str(data[i]).replace("'", "''") for i in data}
        self.execute(f"INSERT INTO {table_name} ({', '.join(data.keys())}) VALUES ({', '.join(data.values())})")

    async def delete(self, table_name: str, pk_column: str, primary_key):
        self.execute("DELETE FROM %s WHERE %s=%s" % (table_name, pk_column, primary_key))

    async def count(self, table_name: str) -> int:
        return len(await self.get_all_primary_keys(table_name))

    async def get_all_primary_keys(self, table_name: str) -> list:
        return [i[0] for i in self.execute('SELECT * FROM %s' % table_name)]

    async def get_all_data(self, table_name: str) -> list[dict]:
        return [getattr(x, "_asdict")() for x in self.execute('SELECT * FROM %s' % table_name)]

    async def change_column_type(self, table_name: str, column_name: str, column_type: str):
        self.execute('ALTER TABLE %s ALTER COLUMN %s TYPE %s' % (table_name, column_name, column_type))

    async def add_column(self, table_name: str, column_name: str, column_type: str):
        self.execute('ALTER TABLE %s ADD %s %s' % (table_name, column_name, column_type))

    async def drop_column(self, table_name: str, column_name: str):
        self.execute('ALTER TABLE %s DROP COLUMN %s' % (table_name, column_name))

    async def drop_table(self, table_name: str,):
        self.execute('DROP TABLE %s' % table_name)

    async def columns(self, table_name: str) -> list[tuple[str]]:
        return [x[0] for x in self.execute("SELECT column_name FROM information_schema.columns WHERE table_schema = 'public' AND table_name = '%s'" % table_name)]

    async def list_tables(self) -> list:
        return inspect(self).get_table_names()

    async def has_table(self, table_name: str) -> bool:
        inspect(self.engine)
        return inspect(self.engine).has_table(table_name)


class Database:
    conn: Connection

    def __init__(self, url: str = os.getenv("DATABASE_URL")):
        """Initiate an SQL database. If url is not passed, environment variable DATABASE_URL is used"""
        if not url:
            logger.critical("No database url found")
            raise SystemExit
        self.engine = create_engine(url, executemany_mode='values_plus_batch')
        self.metadata = MetaData()

    def create(self) -> Connection:
        """Create the database tables and connect to the database.

        Returns:
            Connection object with monkey-patched helper methods.
        """
        self.metadata.create_all(bind=self.engine, checkfirst=True)
        connection = self.engine.connect()
        self.conn = connection
        return connection



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from pyrogram.types import Message
from pystark import Stark, filters
from pystark.plugins.helpers import db


@Stark.cmd(filters=~filters.service, group=2)
async def users_sql(_, msg: Message):
    if msg.from_user:
        q = await db.get("users", msg.from_user.id)
        if not q:
            await db.set("users", msg.from_user.id)


@Stark.cmd('stats', owner_only=True)
async def stats(_, msg: Message):
    users = await db.count("users")
    await msg.reply(f"Total Users : {users}", quote=True)



import json


class PrettyDict(dict):
    def __init__(self, data: dict = None):
        if data:
            for i in data:
                if isinstance(data[i], dict):
                    data[i] = PrettyDict(data[i])
                self[i] = data[i]
        super().__init__()

    def __str__(self):
        return json.dumps(self, indent=4, default=str)

    @property
    def self(self):
        return self



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from pystark import Stark


@Stark.cmd('id', description="Get your Telegram ID")
async def id_func(_, msg):
    if msg.chat.type == 'private':
        await msg.react("Your ID is `{}`".format(msg.from_user.id))
    else:
        if msg.reply_to_message:
            await msg.react("{}'s ID is `{}`".format(msg.reply_to_message.from_user.first_name, msg.reply_to_message.from_user.id))
        else:
            await msg.react("Chat ID is `{}` \n\nYour ID is `{}`".format(msg.chat.id, msg.from_user.id))



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

import os
from pystark import Stark, Message


@Stark.cmd("json", owner_only=True)
async def json(_, msg: Message):
    if not msg.reply_to_message:
        await msg.reply("Please reply to a message", quote=True)
        return
    data = str(msg.reply_to_message)
    if len(data) > 4096:
        with open("message.json", "w", encoding="utf-8") as f:
            f.write(data)
        await msg.reply_document("message.json", quote=True)
        os.remove("message.json")
    else:
        await msg.react(data)


@Stark.cmd("jsondoc", owner_only=True)
async def jsondoc(_, msg: Message):
    if not msg.reply_to_message:
        await msg.reply("Please reply to a message", quote=True)
        return
    data = str(msg.reply_to_message)
    with open("message.json", "w", encoding="utf-8") as f:
        f.write(data)
    await msg.reply_document("message.json", quote=True)
    os.remove("message.json")



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from .. import env
from ..logger import logger
from typing import Union, List
from pyrogram import filters as f
from pyrogram.methods.decorators.on_inline_query import OnInlineQuery


class Inline(OnInlineQuery):
    @staticmethod
    def inline(
        query: Union[str, List[str]] = None,
        startswith: bool = False,
        owner_only: bool = False,
        sudo_only: bool = False,
        group: int = 0,
        filters=None,
        # description: str = None,
        # allow_for_all: bool = False,
        # allow_devs: bool = False,
        # private_only: bool = False,
        # group_only: bool = False,
        # channel_only: bool = False,
        # disable_sudo: bool = False
    ):
        """This decorator is used to handle inline queries. All arguments are optional.

        Parameters:

            query (str | list[str], optional): Query on which your function is called. Pass a list to handle multiple queries. Defaults to None, to handle all queries if your function handles all or if you are using other 'filters'

            startswith (bool, optional): Set to True if you want your function to handle all queries starting with the 'query' string passed. Defaults to False.

            owner_only (bool, optional): Allow only owner to use this command. Defaults to False.

            sudo_only (bool, optional): Allow only sudos to use this command. Includes owner as sudo automatically. Defaults to False.

            group (int, optional): Define a group for this handler. Defaults to 0. [Read More](https://docs.pyrogram.org/topics/more-on-updates#handler-groups)

            filters (pyrogram.filters, optional): Extra filters to apply in your function. Import ``filters`` from pyrogram or pystark to use this. See example below.

        Examples:

            ```python
            from pystark import Stark

            # The normal and easiest way.
            # Bot will show results / execute function, if 'hello' is searched by anyone.
            @Stark.inline('hello')

            # Handle multiple inline queries in one function. Mainly used to execute same function or do some other pythonic thing, like if-else loop.
            # Bot will show results / execute function, if 'hello' or 'hey' is searched by anyone.
            @Stark.inline(['hello', 'hey'])

            # Function will only be triggered if owner searches 'hello', that is, the user whose id is set as OWNER_ID in environment variables.
            # Others will be ignored.
            @Stark.inline('hello', owner_only=True)

            # Function will only be triggered if sudo users or owner searches 'hello', that is, users set as SUDO_USERS or OWNER_ID in environment variables.
            # Others will be ignored.
            @Stark.inline('hello', sudo_only=True)

            # Filter/Handle all queries.

            # Use positive integer to execute after executing another function in default group that also filtered this query.
            @Stark.inline(group=1)

            # or Use negative integer to execute before executing another function in default group that also filtered this query.
            @Stark.inline(group=-1)

            # Don't use this as other functions that handle queries won't work.
            @Stark.inline()

            # Filter other type of queries using 'filters' argument.

            # Import filters from pyrogram or pystark.
            from pystark import filters

            # Filter only queries by 'StarkProgrammer' and 'Designatory'.
            @Stark.inline(filters=filters.user(['StarkProgrammer', 'Designatory']))

            # Filter only queries done in 'StarkBotsChat'
            @Stark.inline(filters=filters.chat('StarkBotsChat'))

            # Filter only queries ending with the word 'baby'.
            @Stark.inline(filters=filters.regex(r'baby$'))

            # Filter all queries with the word 'hello' AND which are done in 'StarkBotsChat'.
            @Stark.inline(filters=filters.chat('StarkBotsChat') & filters.regex('hello'))
            # or
            @Stark.inline('hello', filters=filters.chat('StarkBotsChat'))

            # Filter all queries with the word 'bots' OR which are done in 'StarkBotsChat'
            @Stark.inline(filters=filters.chat('StarkBotsChat') | filters.regex('hello'))

            # Filter all queries with the word 'bots' BUT which are NOT done in 'StarkBotsChat'
            @Stark.inline(filters=~filters.chat('StarkBotsChat') & filters.regex('hello'))
            # or
            @Stark.inline(filters=filters.regex('hello') & ~filters.chat('StarkBotsChat'))
            ```
        """
        # ToDo:
        #   case_sensitive argument
        if isinstance(query, list):
            cmd_filter = f.create(lambda _, __, query_: query_.query.lower() in query)
        elif isinstance(query, str):
            query = query.lower()
            if not startswith:
                cmd_filter = f.create(lambda _, __, query_: query_.query.lower() == query)
            else:
                cmd_filter = f.create(lambda _, __, query_: query_.query.lower().startswith(query))
        elif not query:
            cmd_filter = None
        else:
            logger.warn(f'Inline query cannot be of type {type(query)} - {query}]')
            return
        if filters:
            filters_ = cmd_filter & filters
        else:
            filters_ = cmd_filter
        if sudo_only:
            filters_ = filters_ & f.user(env.SUDO_USERS)
        elif owner_only:
            filters_ = filters_ & f.user(env.OWNER_ID)
        decorator = OnInlineQuery.on_inline_query(filters_, group)
        return decorator

    il = inline  # alias



import inspect
import functools
from pyrogram import filters as f
from pystark.types import PrettyDict
from pyrogram.handlers import MessageHandler
from pyrogram.methods.decorators.on_message import OnMessage
from pyrogram.errors import PeerIdInvalid, BotInlineDisabled
from pyrogram.types import InlineKeyboardMarkup, InlineKeyboardButton
from pystark.env import CMD_PREFIXES, BOT_CMD_PREFIXES, SUDO_USERS, OWNER_ID


command_data = PrettyDict({"total_commands": 0, "all_commands": {}, "sudo_commands": [], "plugins_cmds": {}, "help_strings": {}})
bot_command_data = PrettyDict({"total_commands": 0, "all_commands": {}, "sudo_commands": [], "plugins_cmds": {}, "help_strings": {}})


class Command(OnMessage):
    @staticmethod
    def cmd(
        cmd: str | list[str] = None,
        description: str = None,
        group: int = 0,
        owner_only: bool = False,
        sudo_only: bool = True,
        private_only: bool = False,
        group_only: bool = False,
        channel_only: bool = False,
        regex: str = None,
        filters=None,
    ):
        return _command(
            is_bot=False,
            cmd=cmd,
            description=description,
            group=group,
            owner_only=owner_only,
            sudo_only=sudo_only,
            private_only=private_only,
            group_only=group_only,
            channel_only=channel_only,
            regex=regex,
            filters=filters,
        )

    @staticmethod
    def botcmd(
        cmd: str | list[str] = None,
        description: str = None,
        group: int = 0,
        owner_only: bool = False,
        sudo_only: bool = True,
        private_only: bool = False,
        group_only: bool = False,
        channel_only: bool = False,
        regex: str = None,
        filters=None,
    ):
        return _command(
            is_bot=True,
            cmd=cmd,
            description=description,
            group=group,
            owner_only=owner_only,
            sudo_only=sudo_only,
            private_only=private_only,
            group_only=group_only,
            channel_only=channel_only,
            regex=regex,
            filters=filters,
        )

    command = cmd  # alias
    bot_command = botcmd  # alias


def _command(
    is_bot: bool,
    cmd: str | list[str] = None,
    description: str = None,
    group: int = 0,
    owner_only: bool = False,
    sudo_only: bool = True,
    private_only: bool = False,
    group_only: bool = False,
    channel_only: bool = False,
    regex: str = None,
    filters=None,
):
    """This decorator is used to handle messages. Mainly used to create commands. All arguments are  optional.
    You can also use the alias ``Stark.cmd`` instead of ``Stark.command``.

    Parameters:

        cmd (str | list[str], optional): Command(s) that triggers your function. Defaults to None, which is helpful you only want to use filters argument.

        description (str,  optional): Command description to create Bot Menu. Defaults to None. [Read More](/topics/bot-menu)

        group (int, optional): Define a group for this handler. Defaults to 0. [Read More](https://docs.pyrogram.org/topics/more-on-updates#handler-groups)

        owner_only (bool, optional): Allow only owner to use this command. Defaults to False.

        sudo_only (bool, optional): Allow only sudos to use this command. Includes owner as sudo automatically. Defaults to False.

        private_only (bool, optional): Only handle messages for private chats. Bot will ignore messages in groups and channels. Defaults to False.

        group_only (bool, optional): Only handle messages for groups. Bot will ignore messages in private chats and channels. Defaults to False.

        channel_only (bool, optional): Only handle messages for channels. Bot will ignore messages in private chats and groups. Defaults to False.

        filters (pyrogram.filters, optional): Extra filters to apply in your function. Import ``filters`` from pyrogram or pystark to use this. See example below.

    Examples:

        ```python
        from pystark import Stark

        # The normal way. Bot will reply to command ``/greet`` sent anywhere and by anyone.
        @Stark.command('greet', description='Greet the user')

        # or
        @Stark.cmd('greet', 'Greet the user')

        # Bot will reply only to owner, that is, the user whose id is set as OWNER_ID in environment variables.
        # Others will be ignored.
        @Stark.command('greet', owner_only=True)

        # Bot will reply only to sudo users or owner, that is, users set as SUDO_USERS or OWNER_ID in environment variables.
        # Others will be ignored.
        @Stark.command('greet', sudo_only=True)

        # Bot will reply only if message is sent in private chat (aka pm).
        # Messages in groups and channels will be ignored.
        @Stark.command('greet', private_only=True)

        # Bot will reply only if message is sent in groups.
        # Messages in groups and private chats will be ignored.
        @Stark.command('greet', group_only=True)

        # Bot will reply only if message is sent in channels.
        # Messages in private chats and groups will be ignored.
        @Stark.command('greet', channel_only=True)


        # Filter all messages.

        # Use positive integer to execute after executing another function in default group that also filtered this message.
        @Stark.command(group=1)

        # or Use negative integer to execute before executing another function in default group that also filtered this message.
        @Stark.command(group=-1)

        # Don't use this as other functions won't work.
        @Stark.command()


        # Filter other type of messages using filters.

        # Import filters from pyrogram or pystark.
        from pystark import filters

        # Filter only media messages.
        @Stark.command(filters=filters.media)

        # Filter only text messages.
        @Stark.command(filters=filters.text)

        # Filter only messages sent by 'StarkProgrammer'.
        @Stark.command(filters=filters.user('StarkProgrammer'))

        # Filter only messages sent in 'StarkBotsChat'
        @Stark.command(filters=filters.chat('StarkBotsChat'))

        # Filter only messages with the word 'baby'.
        @Stark.command(filters=filters.regex('baby'))

        # Filter all media messages sent by bots.
        @Stark.command(filters=filters.bot & filters.media)

        # Filter all messages sent by bots except media messages.
        @Stark.command(filters=filters.bot & ~filters.media)

        # Filter either media messages or text messages.
        @Stark.command(filters=filters.text | filters.media)
        ```
    """
    if not cmd:
        sudo_only = False
    if isinstance(cmd, str):
        cmd = [cmd]
    if (owner_only or sudo_only) and cmd:
        command_data["sudo_commands"] += cmd
    if cmd and not filters:
        filters = None
    # else:
    #     filters = filters & ~f.edited
    prefixes = BOT_CMD_PREFIXES if is_bot else CMD_PREFIXES
    data = bot_command_data if is_bot else command_data
    if not cmd and not filters:
        filters_ = f.all
    elif cmd and filters:
        for c in cmd:
            if c not in data["all_commands"]:
                data["total_commands"] += 1
                data["all_commands"][c] = ""
        if cmd and regex:
            filters_ = (f.command(cmd, prefixes=prefixes) | f.regex(regex)) & filters
        else:
            filters_ = f.command(cmd, prefixes=prefixes) & filters
    elif filters:
        if regex:
            filters_ = f.regex(regex) & filters
        else:
            filters_ = filters
    else:
        for c in cmd:
            if c not in data["all_commands"]:
                data["total_commands"] += 1
                data["all_commands"][c] = ""
        if cmd and regex:
            filters_ = f.command(cmd, prefixes=prefixes) | f.regex(regex)
        else:
            filters_ = f.command(cmd, prefixes=prefixes)
    if cmd and description:
        for c in cmd:
            data["all_commands"][c] = description
    if sudo_only:
        filters_ = filters_ & f.user(SUDO_USERS)
    elif owner_only:
        filters_ = filters_ & f.user(OWNER_ID)
    if private_only:
        filters_ = filters_ & f.private
    if group_only:
        filters_ = filters_ & f.group
    if channel_only:
        filters_ = filters_ & f.channel

    def decorator(func):
        plug = inspect.getsourcefile(func)
        if "decorators" in plug:
            plug = func.__filepath__
        if "plugins" in plug:
            x = "plugins"
        else:
            x = "assistant"
        plug = plug.rsplit(x, 1)[1][1:-3]
        plug = plug.replace("\\", "/")
        if cmd:
            if "/" not in plug:
                plug = "others/"+plug
            directory, plug = plug.split("/", 1)
            if not data["plugins_cmds"].get(directory):
                data["plugins_cmds"][directory] = {}

            if data["plugins_cmds"][directory].get(plug):
                data["plugins_cmds"][directory][plug].append(cmd)
            else:
                data["plugins_cmds"][directory][plug] = [cmd]
            # data["help_strings"][cmd[0]] = get_doc(func)

        @functools.wraps(func)
        async def wrapper(*args):
            msg = args[1]
            try:
                await func(msg)
            except PeerIdInvalid:
                if cmd:
                    await msg.tell(
                        "Please PM me so I really know you!",
                        reply_markup=InlineKeyboardMarkup([
                            [InlineKeyboardButton("Start Me", url=f"https://t.me/{args[0].username[1:]}?start=")]
                        ])
                    )
                else:
                    await msg.tell("PEER_ID_INVALID")
            except BotInlineDisabled:
                await msg.tell("Inline Mode is currently disabled. Please turn it on using @BotFather!")

        if not hasattr(func, "handlers"):
            wrapper.handlers = []
        wrapper.handlers.append((MessageHandler(wrapper, filters_), group))

        return wrapper

    return decorator



import os
from .logger import logger
from dotenv import load_dotenv
from pystark import settings as s
from importlib import import_module


load_dotenv(".env")

# API_ID = os.environ.get("API_ID", "").strip()
# API_HASH = os.environ.get("API_HASH", "").strip()
# BOT_TOKEN = os.environ.get("BOT_TOKEN", "").strip()
BOT_SESSION = os.environ.get("BOT_SESSION", "").strip()
SESSIONS = os.environ.get("SESSIONS", "").strip()
if SESSIONS:
    SESSIONS = SESSIONS.split(" ")
OWNER_ID = os.environ.get("OWNER_ID", "").strip().split(" ")
LOG_CHAT = os.environ.get("LOG_CHAT", "").strip()
# SONGS_CHAT = os.environ.get("SONGS_CHAT", "").strip()
SUDO_USERS = os.environ.get("SUDO_USERS", "").strip().split(" ")
MONGO_URL = os.environ.get("MONGO_URL", "").strip()
ARQ = os.environ.get("ARQ", "").strip()
CMD_PREFIXES = list(os.environ.get("CMD_PREFIXES", "").strip())
while "" in CMD_PREFIXES:
    CMD_PREFIXES.remove("")
if not CMD_PREFIXES:
    CMD_PREFIXES = ["."]

BOT_CMD_PREFIXES = list(os.environ.get("BOT_CMD_PREFIXES", "").strip())
while "" in BOT_CMD_PREFIXES:
    BOT_CMD_PREFIXES.remove("")
if not BOT_CMD_PREFIXES:
    BOT_CMD_PREFIXES = ["/"]

# if not API_ID:
#     logger.critical("No API_ID found. Exiting...")
#     raise SystemExit
# if not API_HASH:
#     logger.critical("No API_HASH found. Exiting...")
#     raise SystemExit
if not SESSIONS:
    logger.critical("No session (SESSIONS) found. Exiting...")
    raise SystemExit
if not BOT_SESSION:
    logger.critical("No BOT_SESSION found. Exiting...")
    raise SystemExit
if not LOG_CHAT:
    LOG_CHAT = 0

# try:
#     API_ID = int(API_ID)
# except ValueError:
#     logger.critical("API_ID is not a valid integer. Exiting...")
#     raise SystemExit

owners = []
for o in OWNER_ID:
    if isinstance(o, str) and o.isdigit():
        owners.append(int(o))
    else:
        owners.append(o)
OWNER_ID = owners

sudos = []
for o in SUDO_USERS:
    if o.isdigit():
        sudos.append(int(o))
    else:
        sudos.append(o)
SUDO_USERS = sudos

# LOG_CHAT = -1001567003949  # Stark Bots Logs
try:
    LOG_CHAT = int(LOG_CHAT)
except ValueError:
    logger.critical("LOG_CHAT is not a valid integer. Exiting...")
    raise SystemExit

# try:
#     SONGS_CHAT = int(SONGS_CHAT)
# except ValueError:
#     logger.critical("SONGS_CHAT is not a valid integer. Exiting...")
#     raise SystemExit


default = None


def get_settings():
    global default
    if default:
        return default
    if os.path.exists('settings.py'):
        mod = __import__("settings")
    else:
        # logger.warn("Settings file not found. Default values will be used.")
        mod = import_module('pystark.settings')
    if "DIRECTORY" in mod.__dict__:
        dire = getattr(mod, "DIRECTORY")
        if "PLUGINS" not in mod.__dict__:
            setattr(mod, "PLUGINS", f"{dire}/plugins")
        if "BOT_PLUGINS" not in mod.__dict__:
            setattr(mod, "BOT_PLUGINS", f"{dire}/assistant")
        if "LOCALIZATION" not in mod.__dict__:
            setattr(mod, "LOCALIZATION", f"{dire}/localization")
    if CMD_PREFIXES:
        setattr(mod, "CMD_PREFIXES", CMD_PREFIXES)
    if BOT_CMD_PREFIXES:
        setattr(mod, "BOT_CMD_PREFIXES", BOT_CMD_PREFIXES)
    for i in dir(s):
        if not i.isupper():
            continue
        try:
            getattr(mod, i)
        except AttributeError:
            setattr(mod, i, getattr(s, i))
    prefixes = getattr(mod, "CMD_PREFIXES")
    if not isinstance(prefixes, list):
        setattr(mod, "CMD_PREFIXES", [prefixes])

    os.environ["TIMEZONE"] = getattr(mod, 'TIMEZONE')
    default = mod  # Cache
    return mod


class Settings:
    DIRECTORY: str
    PLUGINS: str | list[str]
    BOT_PLUGINS: str | list[str]
    SET_BOT_MENU: bool
    CMD_PREFIXES: list[str]
    BOT_CMD_PREFIXES: list[str]
    # ADDONS: list[str]
    LOCALIZATION: str
    START: str
    HELP: str
    ABOUT: str
    TIMEZONE: str
    START_IN_GROUPS: str


settings: Settings = get_settings()



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

import functools
from pyrogram.errors import ChatAdminInviteRequired, UserAdminInvalid


aliases = {
    'edit': 'can_be_edited',
    'manage': 'can_manage_chat',
    'delete': 'can_delete_messages',
    'restrict': 'can_restrict_members',
    'promote': 'can_promote_members',
    'change': 'can_change_info',
    'invite': 'can_invite_users',
    'pin': 'can_pin_messages',
    'vc': 'can_manage_voice_chats',
}


class Admins:
    @staticmethod
    def admins(alias: str = "", reply: bool = False):
        def first_decorator(func):
            if alias and alias in aliases:
                perm = aliases[alias]
            elif alias in list(aliases.values()):
                perm = alias
            else:
                perm = None

            @functools.wraps(func)
            async def wrapper(msg):
                bot = msg.c
                bot_status = (await msg.chat.get_member(bot.id))
                if bot_status.status != 'administrator':
                    if reply:
                        await msg.tell("I_AM_NOT_ADMIN")
                    return
                if perm and not bot_status[perm]:
                    if reply:
                        await msg.tell("I_DONT_HAVE_RIGHT", format=perm)
                    return
                try:
                    await func(msg)
                except ChatAdminInviteRequired:
                    if reply:
                        await msg.tell("USER_NOT_PRESENT")
                except UserAdminInvalid:
                    if reply:
                        await msg.tell("NOT_ENOUGH_RIGHTS")
            wrapper.__filepath__ = func.__code__.co_filename
            return wrapper

        return first_decorator



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


from pystark.logger import logger


def patch(obj):
    """Decorator to add new or override existing methods and attributes of an object.
    By default, it'll patch all attributes and functions to the object (generally class) except magic attributes and functions (attributes starting and ending with double-underscores like `__patch__`).

    - Use attribute `__no_patch__` to list all attributes and function names you don't want to patch.
    - Use attribute `__patch__` to list the only attributes and function names you want to patch.
    - Don't specify anything to patch all
    """
    try:
        patch_list = getattr(obj, "__patch__")
        if not isinstance(patch_list, list):
            logger.warn("__patch__ attribute must be a list. Skipping patch..")
            return
    except AttributeError:
        patch_list = []
    try:
        no_patch_list = getattr(obj, "__no_patch__")
        if not isinstance(no_patch_list, list):
            logger.warn("__no_patch__ attribute must be a list. Skipping patch..")
            return
    except AttributeError:
        no_patch_list = []

    def is_patchable(name):
        if name in no_patch_list:
            return False
        elif patch_list and name not in patch_list:
            return False
        return True

    def wrapper(container):
        attrs = vars(container)
        for name in attrs:
            attr = attrs[name]
            if name.startswith("__") and name.endswith("__"):
                continue
            if is_patchable(name):
                old = getattr(obj, name, None)
                setattr(obj, 'old_' + name, old)
                setattr(obj, name, attr)
        return container

    return wrapper



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from pystark import Stark, Message


@Stark.cmd("sudo", owner_only=True)
async def sudo(bot: Stark, msg: Message):
    text = "Sudo Commands Available: \n"
    num = 0
    for i in bot.sudo_commands:
        num += 1
        text += f"\n{num}) /{i}"
    await msg.react(text)



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from pystark import Stark
from pystark.plugins.stark import HOME_BUTTON
from pyrogram.types import InlineKeyboardMarkup
from pystark.plugins.helpers import replace, send_buttons, module, LOADED_BUT_EMPTY, replace_commands


@Stark.cmd('help', description="How to use the bot?", private_only=True)
async def help_func(bot: Stark, msg):
    try:
        if not module.HELP:
            Stark.log(LOADED_BUT_EMPTY.format("help", "help"), "warn")
            return
        text = str(module.HELP)
        if "{commands}" in text:
            text = await replace_commands(bot, text)
        text = await replace(text, msg, bot)
        if await send_buttons():
            await msg.react(text, reply_markup=InlineKeyboardMarkup(HOME_BUTTON))
        else:
            await msg.react(text)
    except AttributeError:
        pass



import aiohttp
from aiohttp import ClientResponse


aio = aiohttp.ClientSession()


class Request:
    @staticmethod
    async def request(method: str, url: str, get_json: bool = True, **kwargs) -> dict[str] | ClientResponse:
        if method.lower() == "get":
            response = await aio.get(url, **kwargs)
        else:
            response = await aio.post(url, **kwargs)
        response.raise_for_status()
        if get_json:
            return await response.json()
        else:
            return response

    async def get(self, url: str, params: dict = None, get_json: bool = True, **kwargs):
        return await self.request("get", url, params=params, get_json=get_json, **kwargs)

    async def post(self, url: str, json: dict = None, data: str | dict = None, get_json: bool = True, **kwargs):
        return await self.request("post", url, json=json, data=data, get_json=get_json, **kwargs)


request = Request()



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


import os
import yaml
from pystark.env import settings
from pystark.logger import logger


file = ""
folder = ""


def l10n_setup():
    global file
    global folder
    folder = settings.LOCALIZATION
    if folder:
        try:
            files = os.listdir(folder)
        except FileNotFoundError:
            logger.critical("Your localization path does not exist. Please correct it. Exiting...")
            raise SystemExit
        sup = ["english.yaml", "english.yml", "en.yaml", "en.yml"]
        for s in sup:
            if s in files:
                file = f"{folder}/{s}"
                break
        if not file:
            f = [f for f in files if f.endswith('.yaml') or f.endswith(".yml") or f.endswith('.json')]
            if f:
                file = f"{folder}/{f[0]}"
            else:
                logger.warn("Your localization directory does not have yaml file")
        return True
    else:
        logger.critical("No localization directory found.")
        raise SystemExit


async def l10n(lang: str = None):
    if not file:
        return
    if not lang:
        current_file = file
    else:
        current_file = ""
        for fi in os.listdir(folder):
            if lang in fi:
                current_file = fi
                break
        if not current_file:
            logger.warn(f"Lang '{lang}' not found. Choosing default.")
            current_file = file
    if current_file.endswith(".yaml") or current_file.endswith(".yml"):
        return await yaml_load(current_file)


def yaml_load(f):
    with open(f, "r", encoding="utf-8") as fi:
        return yaml.safe_load(fi)


def get_all_langs(only_list: bool = False):
    l10n_setup()
    if not file:
        return [] if only_list else {}
    langs = {}
    for fi in os.listdir(folder):
        if fi.endswith(".yaml") or fi.endswith(".yml"):
            data = yaml_load(folder+"/"+fi)
        else:
            continue
        langs[fi.rsplit(".", 1)[0]] = data
    if only_list:
        langs = list(langs.keys())
    return langs



import os
import asyncio
from .user import User
from .chat import Chat
from ..client import Stark
from pystark.utils import patch
from pyrogram.errors import MessageTooLong
from pyrogram.types import Message, MessageEntity


@patch(Message)
class Message(Message):
    _client: Stark
    from_user: User
    chat: Chat

    async def tell(
        self,
        text: str,
        format: str | tuple = None,
        del_in: int = 0,
        quote: bool = True,
        parse_mode: str | None = object,
        entities: list["MessageEntity"] = None,
        disable_web_page_preview: bool = True,
        disable_notification: bool = None,
        reply_to_message_id: int = None,
        schedule_date: int = None,
        reply_markup=None
    ) -> "Message":
        try:
            langs_data = self._client.langs_data
            if text in langs_data["en"]:
                text = langs_data["en"][text]
                if format:
                    if isinstance(format, str):
                        format = (format,)
                    text = text.format(*format)
            if self.from_user.is_self:
                reply = await self.edit(
                    str(text),
                    parse_mode=parse_mode,
                    entities=entities,
                    disable_web_page_preview=disable_web_page_preview,
                    reply_markup=reply_markup,
                )
            else:
                reply = await self.reply(
                    str(text),
                    quote=quote,
                    parse_mode=parse_mode,
                    entities=entities,
                    disable_notification=disable_notification,
                    reply_to_message_id=reply_to_message_id,
                    schedule_date=schedule_date,
                    disable_web_page_preview=disable_web_page_preview,
                    reply_markup=reply_markup,
                )
        except MessageTooLong:
            reply = await self.reply(
                "Sending as document...",
                quote=quote,
                parse_mode=parse_mode,
                entities=entities,
                disable_notification=disable_notification,
                reply_to_message_id=reply_to_message_id,
                schedule_date=schedule_date,
                disable_web_page_preview=disable_web_page_preview,
                reply_markup=reply_markup,
            )
            file = f'{reply.message_id}.txt'
            with open(file, 'w+', encoding="utf-8") as f:
                f.write(text)
            await reply.delete()
            reply = await self.reply_document(
                document=file,
                caption="Output",
                quote=quote,
                parse_mode=parse_mode,
                caption_entities=entities,
                disable_notification=disable_notification,
                reply_to_message_id=reply_to_message_id,
                schedule_date=schedule_date,
                reply_markup=reply_markup,
            )
            os.remove(file)
        if del_in:
            await asyncio.sleep(del_in)
            await reply.delete()
        return reply

    @property
    def args(self, split: str = " ") -> list[str]:
        """List arguments passed in a message. Removes first word (the command itself)

        Parameters:

            split (str, optional): Define how to split the arguments, defaults to whitespace.

        Example:

            If text is `/start reply user`, return value would be `["reply", "user"]`
        """
        args: list[str] = self.text.markdown.split(split)
        args.pop(0)
        if args:
            args[0] = args[0].strip()
            if "\n" in args[0]:
                wtf = args[0]
                f, s = wtf.split("\n", 1)
                args[0] = f
                args.insert(1, s)
        return args

    @property
    def input(self) -> str | None:
        """Input passed in a message. Removes first word (the command itself)

        Example:

            If text is `/start reply user`, return value would be `reply user`
        """
        i = self.text.markdown.split(" ", 1)
        if len(i) > 1 and i[1]:
            return i[1]
        return

    @property
    def ref(self) -> int | str | None:
        """Returns the referred user's id or username. To get the full user, use method `get_ref_user`

        Useful to get the referent user's id or username for a command.
        If command was replied to a message, the replied message's user id is returned.
        Otherwise, the first argument of command is returned (which isn't guaranteed to be an actual user or an id).

        **Example**:

            If command `/ban` was replied to a user's message, user id (`message.reply_to_message.from_user.id`) is returned.

            If command `/ban 12345678` was sent, user id (integer) `12345678` is returned.

            If command `/ban StarkProgrammer` was sent, username (string) `StarkProgrammer` is returned.

            If command `/ban 87654321 12345678` was sent, user id (integer) `87654321` is returned but `12345678` is ignored because it's not the first argument.

            If command `/ban` was sent with no arguments and was also not a reply to any message, None is returned.

            If command `/ban ok` was sent, "ok" is returned which isn't an actual user but will not raise exception.
        """
        if self.reply_to_message:
            return self.reply_to_message.from_user.id
        args = self.args
        if not args:
            return
        return args[0] if not args[0].isdigit() else int(args[0])

    async def get_ref_user(self) -> User | None:
        """Returns the full referred user. To get only user id or username, use property `ref` as it's faster.

        Useful to get the referent of a command.
        If command was replied to a message, the replied message's user is returned.
        Otherwise, the first argument of command is used to get the user.

        !!! note

            If command is not replied then first argument of command is considered user id or username. It's on you to handle PeerIdInvalid and UsernameInvalid

        **Example**:

            If command `/ban` was replied to a user's message, user (`message.reply_to_message.from_user`) is returned.

            If command `/ban 12345678` was sent, the user instance of user with id `12345678` is returned.

            If command `/ban StarkProgrammer` was sent, the user instance of user with username `StarkProgrammer` is returned.

            If command `/ban 87654321 12345678` was sent, the user instance of user with id `87654321` is returned but `12345678` is ignored because it's not the first argument.

            If command `/ban` was sent with no arguments and was also not a reply to any message, None is returned.
        """
        if self.reply_to_message:
            return self.reply_to_message.from_user
        args = self.args
        if not args:
            return
        user = args[0] if not args[0].isdigit() else int(args[0])
        user = await self._client.get_users(user)
        return user

    async def get_ref_chat(self) -> Chat | None:
        args = self.args
        if not args:
            return
        chat = args[0] if not args[0].isdigit() else int(args[0])
        chat = await self._client.get_chat(chat)
        return chat

    async def get_aor(self) -> str:
        """Get arg or reply text"""
        if self.reply_to_message:
            return self.reply_to_message.text.markdown
        else:
            return self.input

    # async def extract_data(self, string: str = "", replace: bool = True, user=None) -> (str, InlineKeyboardMarkup):
    #     if not string:
    #         if self.reply_to_message:
    #             string = self.reply_to_message.text.markdown if self.reply_to_message.text else self.reply_to_message.caption.markdown
    #         else:
    #             string = self.text.markdown.split(" ", 1)[-1]
    #     text, buttons = await extract_msg_data(string)
    #     if not user:
    #         user = self.from_user
    #     if replace:
    #         if "{first}" in text:
    #             text = text.replace("{first}", user.first_name)
    #         if "{last}" in text:
    #             if user.last_name:
    #                 text = text.replace("{last}", user.last_name)
    #             else:
    #                 if "{last} " in text:
    #                     text = text.replace("{last} ", "")
    #                 elif " {last}" in text:
    #                     text = text.replace(" {last}", "")
    #                 else:
    #                     text = text.replace("{last} ", "")
    #         if "{fullname}" in text:
    #             if user.last_name:
    #                 name = user.first_name + " " + user.last_name
    #             else:
    #                 name = user.first_name
    #             text = text.replace("{fullname}", name)
    #         if "{username}" in text:
    #             text = text.replace("{username}", "@"+user.username if user.username else user.mention)
    #         if "{mention}" in text:
    #             text = text.replace("{mention}", user.mention)
    #         if "{chatid}" in text:
    #             text = text.replace("{chatid}", str(self.chat.id))
    #         if "{userid}" in text:
    #             text = text.replace("{userid}", str(user.id))
    #         if "{id}" in text:  # {userid} alias
    #             text = text.replace("{id}", str(user.id))
    #         if "{chatname}" in text:
    #             text = text.replace("{chatname}", self.chat.title)
    #         if "{chat}" in text:  # {chatname} alias
    #             text = text.replace("{chat}", self.chat.title)
    #         if "{count}" in text:
    #             chat = await self._client.get_chat(self.chat.id)
    #             text = text.replace("{count}", str(chat.members_count))
    #         # if "{rules}" in text:
    #         #     if buttons:
    #         #         buttons = buttons.inline_keyboard.copy()
    #         #         buttons.insert()
    #     return text, buttons

    @property
    def client(self):
        return self._client

    @property
    def c(self):
        return self._client



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.

import functools
from pystark import env
from pystark.logger import logger
from pyrogram import filters as f
from pyrogram.types import CallbackQuery
from pyrogram.handlers import CallbackQueryHandler
from pyrogram.methods.decorators.on_callback_query import OnCallbackQuery


class Callback(OnCallbackQuery):
    @staticmethod
    def callback(
        query: str | list[str] = None,
        startswith: bool = False,
        owner_only: bool = False,
        sudo_only: bool = True,
        group: int = 0,
        filters=None
    ):
        # ToDo:
        #   case_sensitive argument
        if isinstance(query, list):
            cmd_filter = f.create(lambda _, __, query_: query_.data.lower() in query)
        elif isinstance(query, str):
            query = query.lower()
            if not startswith:
                cmd_filter = f.create(lambda _, __, query_: query_.data.lower() == query)
            else:
                cmd_filter = f.create(lambda _, __, query_: query_.data.lower().startswith(query))
        elif not query:
            cmd_filter = None
        else:
            logger.warn(f'Callback query cannot be of type {type(query)} - {query}]')
            return
        if filters:
            filters_ = cmd_filter & filters
        else:
            filters_ = cmd_filter
        if sudo_only:
            filters_ = filters_ & f.user(env.SUDO_USERS)
        elif owner_only:
            filters_ = filters_ & f.user(env.OWNER_ID)

        def decorator(func):
            @functools.wraps(func)
            async def wrapper(*args):
                q = args[1]  # type: CallbackQuery
                if q.from_user.id in env.SUDO_USERS:
                    await func(*args)
                else:
                    await q.answer("You can't use this userbot. I'm sorry!", show_alert=True)

            if not hasattr(func, "handlers"):
                wrapper.handlers = []
            wrapper.handlers.append((CallbackQueryHandler(wrapper, filters_), group))

            return wrapper

        return decorator

    cb = callback  # alias



import os
from pyrogram.types import InlineKeyboardButton, InlineKeyboardMarkup

sep = "$buttons$"


async def extract_msg_data(string: str):
    if sep in string.lower():
        x = string.lower()
        s = x.find(sep)
        e = x.find(sep)+len(sep)
        text = string[:s].strip()
        buttons = string[e:].strip()
    else:
        text = string
        buttons = None
    buttons = await string_to_buttons(buttons)
    return text, buttons


async def string_to_buttons(string) -> None | InlineKeyboardMarkup:
    if not string:
        return
    buttons = []
    rows = string.split('\n')
    for row in rows:
        row_buttons = []
        r_b = row.split("|")
        for b in r_b:
            data = b.split('-', 1)
            if len(data) < 2:
                continue
            text = data[0].strip()
            but = data[1].strip()
            data = but.split(":", 1)
            if len(data) < 2:
                continue
            button_type = data[0].strip()
            ans = data[1].strip()
            if button_type == "alert":
                row_buttons.append(InlineKeyboardButton(text, callback_data=f"alert+{ans}"))
            else:
                row_buttons.append(InlineKeyboardButton(text, url=ans))
        buttons.append(row_buttons)
    if buttons:
        buttons = InlineKeyboardMarkup(buttons)
    return buttons


def get_db_url(i: str):
    i = os.environ.get(i, "").strip()
    if 'postgres' in i and 'postgresql' not in i:
        i = i.replace("postgres", "postgresql")
    if "psycopg2" not in i:
        i = i.replace("postgresql", "postgresql+psycopg2")
    return i


async def list_to_string(data: list, pref: str = None) -> str:
    string = ""
    num = 0
    for i in data:
        num += 1
        if not pref:
            if isinstance(i, int) or (isinstance(i, str) and i.isdigit()):
                i = f"`{i}`"
            string += f"{num}) {i}\n"
        else:
            text = f"{pref}{i}"
            if pref != "/":
                text += f"`{text}`"
            string += text
    return string



import asyncio
import humanize
from pyrogram.errors import FloodWait, MessageNotModified


async def pyrogress(current, total, message, process):
    new_current = humanize.naturalsize(current, binary=True)
    new_total = humanize.naturalsize(total, binary=True)
    # if int(float(new_current.split()[0])) % 10 != 0:
    #     return
    try:
        percentage = round((current * 100) / total, 2)
        try:
            await message.edit(f"**{process}** \n\n**Progress :** {new_current}/{new_total} | {percentage}℅")
        except FloodWait as e:
            await asyncio.sleep(e.value)
        except MessageNotModified:  # Sometimes pyrogram returns same i think
            pass
    except ZeroDivisionError:
        try:
            await message.edit(f"**{process}** \n\n**Progress :** {new_current}")
        except FloodWait as e:
            await asyncio.sleep(e.value)



import asyncio
from datetime import datetime
from pystark.utils import patch
from pystark.env import MONGO_URL
from pymongo.command_cursor import CommandCursor
from motor.motor_asyncio import AsyncIOMotorClient
from motor.core import AgnosticDatabase, AgnosticCollection


@patch(AgnosticCollection)
class AgnosticCollection(AgnosticCollection):
    async def get(self, document: dict) -> dict:
        return await self.find_one(document)

    async def mget(self, document: dict) -> list[dict]:
        ans = []
        async for doc in self.find(document):
            ans.append(doc)
        return ans

    async def set(self, document: dict) -> bool:
        a, b = {}, {}
        for i in document:
            if i in ["chat_id", "name"]:
                c = a
            else:
                c = b
            c[i] = document[i]
        ans = await self.get(a)
        if not ans:
            return await self.insert_one(document)
        else:
            return await self.update_one(a, {"$set": b})

    async def delete(self, document: dict) -> bool:
        return await self.find_one_and_delete(document)

    async def get_all(self) -> list[dict]:
        ans = []
        async for doc in self.find():
            ans.append(doc)
        return ans


class MongoDB:
    def __init__(self, url: str = MONGO_URL):
        self.client = AsyncIOMotorClient(url)
        self.client.get_io_loop = asyncio.get_running_loop
        self.db: AgnosticDatabase = self.client["pystark"]
        self.notes_col: AgnosticCollection = self.db["notes"]
        self.filters_col: AgnosticCollection = self.db["filters"]
        self.lists_col: AgnosticCollection = self.db["lists"]
        self.greetings_col: AgnosticCollection = self.db["greetings"]
        self.data_col: AgnosticCollection = self.db["data"]
        # self.chats_col: AgnosticCollection = self.db["chats"]
        self._default_dbs = ["admin", "local"]
        self._default_response = "value"

    # ---------------------------- Filters --------------------------- #

    async def get_all_filters(self, chat_id: int) -> dict[str, str]:
        return await self._col_get_all(self.filters_col, chat_id)

    async def get_filter(self, chat_id: int, name: str) -> str:
        return await self._col_get(self.filters_col, chat_id, name)

    async def set_filter(self, chat_id: int, name: str, value: str) -> bool:
        return await self._col_set(self.filters_col, chat_id, name, value)

    async def delete_filter(self, chat_id: int, name: str) -> bool:
        return await self._col_delete(self.filters_col, chat_id, name)

    async def delete_all_filters(self, chat_id: int) -> int:
        return await self._col_delete_all(self.filters_col, chat_id)

    # ---------------------------- Notes --------------------------- #

    async def get_all_notes(self, chat_id: int) -> dict[str, str]:
        return await self._col_get_all(self.notes_col, chat_id)

    async def get_note(self, chat_id: int, name: str) -> str:
        return await self._col_get(self.notes_col, chat_id, name)

    async def set_note(self, chat_id: int, name: str, value: str) -> bool:
        return await self._col_set(self.notes_col, chat_id, name, value)

    async def delete_note(self, chat_id: int, name: str) -> bool:
        return await self._col_delete(self.notes_col, chat_id, name)

    async def delete_all_notes(self, chat_id: int) -> int:
        return await self._col_delete_all(self.notes_col, chat_id)

    # ---------------------------- Lists --------------------------- #

    async def get_all_lists(self, chat_id: int) -> dict[str, str]:
        return await self._col_get_all(self.lists_col, chat_id)

    async def get_list(self, chat_id: int, name: str) -> str:
        return await self._col_get(self.lists_col, chat_id, name)

    async def set_list(self, chat_id: int, name: str, value: str) -> bool:
        return await self._col_set(self.lists_col, chat_id, name, value)

    async def delete_list(self, chat_id: int, name: str) -> bool:
        return await self._col_delete(self.lists_col, chat_id, name)

    async def delete_all_lists(self, chat_id: int) -> int:
        return await self._col_delete_all(self.lists_col, chat_id)

    # ---------------------------- Users --------------------------- #

    # async def get_all_users(self) -> list[int]:
    #     return [a["user_id"] for a in await self.users_col.get_all()]
    #
    # async def get_user(self, user_id: int) -> str:
    #     return await self.users_col.get({"user_id": user_id})
    #
    # async def set_user(self, user_id: int) -> bool:
    #     return await self.users_col.set({"user_id": user_id})
    #
    # async def count_users(self) -> int:
    #     return len(await self.users_col.get_all())
    #
    # async def delete_user(self, chat_id: int, name: str) -> bool:
    #     return await self._col_delete(self.users_col, chat_id, name)

    # ---------------------------- Chats --------------------------- #

    # async def get_all_chats(self) -> list[int]:
    #     return [a["chat_id"] for a in await self.chats_col.get_all()]
    #
    # async def get_chat(self, chat_id: int) -> str:
    #     return await self.chats_col.get({"chat_id": chat_id})
    #
    # async def set_chat(self, chat_id: int) -> bool:
    #     return await self.chats_col.set({"chat_id": chat_id})
    #
    # async def count_chats(self) -> int:
    #     return len(await self.chats_col.get_all())
    #
    # async def delete_chat(self, chat_id: int, name: str) -> bool:
    #     return await self._col_delete(self.chats_col, chat_id, name)

    # ---------------------------- Special --------------------------- #

    # ------------------------ Sudos ----------------------- #

    async def list_sudo(self) -> list[int]:
        data = await self.data_col.get({"name": "sudos"})
        if not data:
            await self.data_col.set({"name": "sudos", "value": []})
            return []
        return (await self.data_col.get({"name": "sudos"}))["value"]

    async def add_sudo(self, user_id: int) -> bool:
        sudos = await self.list_sudo()
        sudos.append(user_id)
        return await self.data_col.set({"name": "sudos", "value": sudos})

    async def delete_sudo(self, user_id: int) -> bool:
        sudos = await self.list_sudo()
        if user_id in sudos:
            sudos.remove(user_id)
        return await self.data_col.set({"name": "sudos", "value": sudos})

    # ------------------------ AFKs ----------------------- #

    async def set_afk(self, user_id: int, username: str, reason: str = None) -> bool:
        return await self.data_col.set({"user_id": user_id, "name": "afk", "value": reason, "time": datetime.now(), "username": username})

    async def get_afk(self, user_id: int) -> dict:
        data = await self.data_col.get({"user_id": user_id, "name": "afk"})
        return data

    async def get_all_afk(self) -> list[dict]:
        data = await self.data_col.mget({"name": "afk"})
        return data

    async def reset_afk(self, user_id: int) -> bool:
        return await self.data_col.delete({"user_id": user_id, "name": "afk"})

    # ------------------------ Replicate ----------------------- #

    async def get_replicate(self) -> dict:
        return await self.data_col.get({"name": "replicate"})

    async def set_replicate(self, first_name: str, last_name: str, bio: str, dp: bytes, video: bool) -> bool:
        return await self.data_col.set(
            {"name": "replicate", "first_name": first_name, "last_name": last_name, "bio": bio, "dp": dp, "video": video}
        )

    async def reset_replicate(self) -> bool:
        return await self.data_col.delete({"name": "replicate"})

    # ------------------------ Packs ----------------------- #

    async def get_pack(self, user_id: int, type: str) -> str:
        data = await self.data_col.get({"name": "packs", "user_id": user_id})
        if data:
            data = data.get(type)
        return data

    async def set_pack(self, user_id: int, value: str, type: str) -> bool:
        return await self.data_col.set(
            {"name": "packs", "user_id": user_id, type: value}
        )

    # ------------------------ AI ----------------------- #

    async def ai(self, id: int, delete: bool, user: bool) -> bool:
        if user:
            t = "users"
        else:
            t = "chats"
        q = await self.data_col.get({"name": "ai"})
        li = None
        if q:
            li = q.get(t)
        if not li:
            li = []
        if delete:
            if id in li:
                li.remove(id)
        else:
            if id not in li:
                li.append(id)
        return await self.data_col.set({"name": "ai", t: li})

    async def set_ai(self, id: int, user: bool = True) -> bool:
        return await self.ai(id, False, user)

    async def delete_ai(self, id: int, user: bool = True) -> bool:
        return await self.ai(id, True, user)

    async def get_all_ai(self) -> dict:
        return await self.data_col.get({"name": "ai"})

    # ---------------------------- Greetings --------------------------- #

    async def get_greetings(self, chat_id: int) -> dict:
        return await self.greetings_col.get({"chat_id": chat_id})

    async def set_greetings(
        self,
        chat_id: int,
        welcome: str = None,
        goodbye: str = None,
        welcome_on: bool = None,
        goodbye_on: bool = None,
    ) -> bool:
        doc = {"chat_id": chat_id}
        if welcome:
            doc["welcome"] = welcome
        if goodbye:
            doc["goodbye"] = goodbye
        if welcome_on:
            doc["welcome_on"] = welcome_on
        if goodbye_on:
            doc["goodbye_on"] = goodbye_on
        return await self.greetings_col.set(doc)

    async def delete_greetings(self, chat_id: int, name: str) -> bool:
        return await self._col_delete(self.greetings_col, chat_id, name)

    # ---------------------------- Other --------------------------- #

    async def documents(self):
        ...

    async def total_documents(self) -> int:
        ...

    # ------------------ Database and Collections --------------------- #

    async def database_names(self, remove: bool = True) -> list[str]:
        ans: list[str] = await self.client.list_database_names()
        if remove:
            ans = [f for f in ans if f not in self._default_dbs]
        return ans

    async def databases(self, remove: bool = True) -> list[dict]:
        cursor = await self.client.list_databases()
        ans = await self._iter_cursor(cursor)
        if remove:
            ans = [f for f in ans if f["name"] not in self._default_dbs]
        return ans

    @staticmethod
    async def collection_names(db: AgnosticDatabase) -> list[str]:
        return await db.list_collection_names()

    async def collections(self, db: AgnosticDatabase) -> list[dict]:
        cursor = await db.list_collections()
        return await self._iter_cursor(cursor)

    # ---------------------- Private Methods --------------------- #

    async def _col_get(self, col: AgnosticCollection, chat_id: int, name: str) -> str | None:
        x = await col.get(self._generic_dict(chat_id, name))
        if x:
            return x[self._default_response]
        return

    async def _col_set(self, col: AgnosticCollection, chat_id: int, name: str, value: str) -> bool:
        return await col.set(self._generic_dict(chat_id, name, value))

    async def _col_delete(self, col: AgnosticCollection, chat_id: int, name: str) -> bool:
        return await col.delete(self._generic_dict(chat_id, name))

    async def _col_get_all(self, col: AgnosticCollection, chat_id: int) -> dict[str, str]:
        ans = {}
        async for doc in col.find(self._generic_dict(chat_id)):
            ans[doc["name"]] = doc["value"]
        return ans

    async def _col_delete_all(self, col: AgnosticCollection, chat_id: int) -> int:
        res = await col.delete_many(self._generic_dict(chat_id))
        return res.deleted_count

    # ---------------------------- Cursor --------------------------- #

    async def _iter_cursor(self, cursor) -> list[dict]:
        if isinstance(cursor, CommandCursor):
            d = self._iter_cursor_sync(cursor)
        else:
            d = await self._iter_cursor_async(cursor)
        return d

    @staticmethod
    def _iter_cursor_sync(cursor) -> list[dict]:
        data = []
        try:
            while True:
                data.append(cursor.next())
        except StopIteration:
            pass
        return data

    @staticmethod
    async def _iter_cursor_async(cursor) -> list[dict]:
        data = []
        try:
            while True:
                data.append(await cursor.next())
        except StopAsyncIteration:
            pass
        return data

    @staticmethod
    def _generic_dict(chat_id: int, name: str = "", value: str = "") -> dict:
        ans = {"chat_id": chat_id}
        if name:
            ans["name"] = name
        if value:
            ans["value"] = value
        return ans

    # ---------------------------------------------------------------- #

    async def print(self):
        x = await self.database_names()
        print("Database Names: ", x)
        x = await self.databases()
        print("Databases Info: ", x)
        x = await self.collection_names(self.db)
        print("Collections Names in main database: ", x)
        x = await self.collections(self.db)
        print("Collections in main database: ", x)


mongo = MongoDB()



from .requests import request


async def paste(text: str, bin: str = "haste"):
    if bin.endswith("bin"):
        bin = bin[:-3]
    if bin == "space":
        base = "https://spaceb.in/"
        path = "api/v1/documents"
        res = await request(base+path, data={"content": text, "extension": "txt"})
        link = base+res["payload"]["id"]
    elif bin == "neko":
        base = "https://nekobin.com/"
        path = "api/documents"
        res = await request(base+path, json={"content": text})
        link = base+res["result"]["key"]
    elif bin == "bat":
        base = "https://batbin.me/"
        path = "api/v2/paste"
        res = await request(base+path, data=text)
        if res["success"]:
            link = base+res["message"]
        else:
            link = res["message"]
    else:
        # if not bin == "haste":
        #     logger.log(f"'{bin}' not found. Using 'haste'")
        base = "https://hastebin.com/"
        path = "documents"
        res = await request(base+path, data=text)
        link = base+res["key"]
    return link



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


import functools
from pyrogram.errors import PeerIdInvalid, UsernameInvalid


class Extras:
    @staticmethod
    def ref_required(func):
        """User ID or Reply to User required."""
        @functools.wraps(func)
        async def wrapper(msg):
            try:
                if not msg.reply_to_message and not msg.args:
                    await msg.tell("REF_REQUIRED")
                    return
                await func(msg)
            except UsernameInvalid:
                await msg.tell("USERNAME_INVALID")
            except PeerIdInvalid:
                await msg.tell("PEER_ID_INVALID")

        wrapper.__filepath__ = func.__code__.co_filename
        return wrapper

    @staticmethod
    def arg_required(what: str):
        """Argument required"""
        def decorator(func):
            @functools.wraps(func)
            async def wrapper(msg):
                if not msg.args:
                    await msg.tell("ARG_REQUIRED", format=what)
                    return
                await func(msg)

            wrapper.__filepath__ = func.__code__.co_filename
            return wrapper

        return decorator

    @staticmethod
    def reply_required(func):
        """Reply to a message required"""
        @functools.wraps(func)
        async def wrapper(msg):
            if not msg.reply_to_message:
                await msg.tell("REPLY_REQUIRED")
                return
            await func(msg)

        wrapper.__filepath__ = func.__code__.co_filename
        return wrapper

    @staticmethod
    def aor_required(func):
        """Arg or Reply: Reply to a message or passing an argument required"""
        @functools.wraps(func)
        async def wrapper(msg):
            if not msg.reply_to_message and not msg.args:
                await msg.tell("AOR_REQUIRED")
                return
            await func(msg)

        wrapper.__filepath__ = func.__code__.co_filename
        return wrapper



"""This module contains some sticker conversion tools for Telegram"""

from PIL import Image


async def to_webp(path: str) -> str:
    """Convert any image to webp for telegram static stickers. Resizes according to telegram sticker requirements automatically.
    !!! important
        Requires library `pillow` which is not bundled with pystark.
        Add `pillow` to your `requirements.txt` | `pip install pillow`
    Parameters:
        path: Path of image to convert to webp.
    Returns:
        str: path to the converted webp image.
    Examples:
        ```python
        # This is an example plugin that converts any sent image to sticker and replies it back.
        import os
        from pystark import Stark, filters
        from pystark.helpers.stickers import to_webp
        # filters.photo means that this function will be executed if any message has a photo.
        @Stark.cmd(extra_filters=filters.photo)
        async def reply_webp(_, msg):
            message = await msg.reply("Converting...")
            file = await msg.download()
            # main function
            sticker = await to_webp(file)
            await msg.reply_sticker(sticker)
            await message.delete()
            # remove your image and sticker later
            os.remove(sticker)
            os.remove(file)
        ```
    """
    name = path.rsplit(".", 1)[0]+".webp"
    im = Image.open(path).convert("RGB")
    width, height = im.size
    max_pixels = 512
    ratio = min(max_pixels/width, max_pixels/height)
    size = (int(ratio*width), int(ratio*height))
    im.resize(size)
    im.save(name, "webp")
    return name



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


class Permissions:
    EDIT = 'can_be_edited',
    MANAGE = 'can_manage_chat',
    DELETE = 'can_delete_messages',
    RESTRICT = 'can_restrict_members',
    PROMOTE = 'can_promote_members',
    CHANGE = 'can_change_info',
    INVITE = 'can_invite_users',
    PIN = 'can_pin_messages',
    VC = 'can_manage_voice_chats',



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


import os
import asyncio
import subprocess


def exec_sync(cmd: str, shell: bool = False) -> (str, str):
    """Execute a system command synchronously using Python and get the stdout and stderr as strings.

    Parameters:
        cmd: Command to execute.
        shell: Whether to run in shell mode.

    Returns:
        tuple of stdout and stderr as strings
    """
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=shell)
    stdout, stderr = proc.communicate()
    # None shouldn't be converted to "None" or b''
    if stdout:
        stdout = str(stdout.decode("utf-8"))
    else:
        stdout = ""
    if stderr:
        stderr = str(stderr.decode("utf-8"))
    else:
        stderr = ""
    return stdout, stderr


async def exec_async(cmd: str, shell: bool = False) -> (str, str):
    """Execute a system command asynchronously using Python and get the stdout and stderr as strings

    Parameters:
        cmd: Command to execute.
        shell: Whether to run in shell mode.

    Returns:
        tuple of stdout and stderr as strings
    """
    if shell:
        proc = await asyncio.create_subprocess_shell(cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.STDOUT, shell=True)
    else:
        proc = await asyncio.create_subprocess_shell(cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.STDOUT, shell=True)
    stdout, stderr = await proc.communicate()
    if stdout:
        stdout = str(stdout.decode("utf-8"))
    else:
        stdout = ""
    if stderr:
        stderr = str(stderr.decode("utf-8"))
    else:
        stderr = ""
    return stdout, stderr


async def ffmpeg(old=None, new=None, common_ffmpeg_cmd: bool = False, command: str = None):
    if new and os.path.exists(new):
        return new, None, None
    if common_ffmpeg_cmd:
        command = """ffmpeg -i "{}" -c copy "{}" """
    if old and new:
        command = command.format(old, new)
    process = await asyncio.create_subprocess_shell(
        command,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE
    )
    stdout, stderr = await process.communicate()
    return new, stdout, stderr



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


from .admins import Admins
from .inline import Inline
from .extras import Extras
from .command import Command
from .callback import Callback
from pyrogram.methods import Methods


class Mechanism(Methods, Command, Inline, Callback, Admins, Extras):
    pass



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


import os
import re
from setuptools import setup

with open("README.md", encoding="utf-8") as f:
    long_description = "\n".join([x for x in f.read().split("\n") if not x.startswith('>')])

with open("requirements.txt", encoding="utf-8") as r:
    install_requires = [i.strip() for i in r if not i.startswith('#')]

with open("pystark/constants.py", "r", encoding="utf-8") as f:
    text = f.read()
    pat = r"['\"]([^'\"]+)['\"]"
    version = re.search("__version__ = "+pat, text).group(1)
    # beta_version = re.search("__beta_version__ = "+pat, text).group(1)
    description = re.search("__description__ = "+pat, text).group(1)


def get_packages():
    return [path.replace("\\", ".").replace("/", ".") for path, _, _ in os.walk("pystark") if "__" not in path]


setup(
    name='some-random-name-for-testing',
    packages=get_packages(),
    version=version,
    license='GPLv3+',
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='StarkProgrammer',
    author_email='starkbotsindustries@gmail.com',
    url='https://github.com/StarkBotsIndustries/PyStark',
    keywords=['telegram', 'bot', 'pyrogram', 'python', 'telegram-bot'],
    install_requires=install_requires,
    zip_safe=False,
    python_requires=">=3.10",
    dependency_links=['https://github.com/pyrogram/pyrogram/tarball/master'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Natural Language :: English',
        'Topic :: Communications :: Chat',
        'Topic :: Education :: Testing',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    project_urls={
        "Support": "https://t.me/StarkBotsChat",
        "Community": "https://t.me/StarkBots",
    },
)



from pystark.utils import patch
from pyrogram.types import Chat
from pyrogram.types import ChatMember
# from pystark.others.database import get_chat


@patch(Chat)
class Chat:
    async def get_admins(self) -> list[ChatMember]:
        return await self.get_members(filter="administrators")

    # async def get_lang(self):
    #     chat = await get_chat(self.id)
    #     if chat.get("lang"):
    #         return chat["lang"]
    #     else:
    #         return "en"



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.


import os
import sys
import struct
import logging
import asyncio
import importlib
import importlib.util
from pystark import env
from pytgcalls import PyTgCalls
from pystark.logger import logger
from pystark.database import mongo
from pystark.types import PrettyDict
from pyrogram import Client, idle, raw
from pystark.utils import get_all_langs
from pystark.decorators import Mechanism
from pystark.constants import __version__
from inspect import getmembers, isfunction
from pystark.env import SUDO_USERS, settings, SESSIONS
from pytgcalls.pytgcalls_session import PyTgCallsSession
from pystark.decorators.command import command_data, bot_command_data
from pyrogram.types import InlineKeyboardButton, InlineKeyboardMarkup
from pyrogram.errors import ApiIdInvalid, AccessTokenInvalid, AuthKeyDuplicated, AuthKeyUnregistered, UserDeactivated


__printed__ = False
__buttons__ = {}


class Stark(Client, Mechanism):
    __support__ = "StarkBotsChat"
    __updates__ = "pystark"
    __channel__ = "StarkBots"
    __data__ = {"total_plugins": 0, "all_plugins": {}}
    langs_data = []
    call: "Calls" = None
    bot: "Stark"
    buttons = {}
    # back_button_text = "<= Back <="
    id = 0
    name = ""
    username = ""
    sudos = []

    def __init__(self, session: str = None, is_bot: bool = False, **kwargs):
        global __printed__
        if not __printed__:
            print(f'Welcome to PyStark [v{__version__}]')
            print('Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries> \n')
            self.log(f"Initializing the UserBot")
            __printed__ = True
        self.sep = "\\" if sys.platform.startswith("win") else "/"
        self.is_bot = is_bot
        if not session:
            if not SESSIONS or not SESSIONS[0]:
                self.log("No session passed to Stark class", "warn")
                raise SystemExit
            session = SESSIONS[0]
        name = "bot" if is_bot else "user"
        super().__init__(
            name=name,
            session_string=session,
            in_memory=True,
            **kwargs
        )

    def run(self, start: bool = True):
        """Main method of `Stark` class which loads plugins and activates/runs your bot."""
        t = 'Assistant' if self.is_bot else 'UserBot'
        self.log(f"Starting the {t}")
        if start:
            self._start()
        self._set_info()
        self.sudos = asyncio.get_event_loop().run_until_complete(self.load_sudos())
        self.log(f"Loading {t} Modules")
        if self.is_bot:
            plugs = f"{settings.BOT_PLUGINS}"
        else:
            plugs = f"{settings.PLUGINS}"
        self.load_modules(plugs)
        self.langs_data = {"en": get_all_langs()["en"]}
        global __buttons__
        if not __buttons__:
            __buttons__ = self._buttons_cache()
        self.buttons = __buttons__
        logger.info(f"{self.username} is now running...")

    def _start(self):
        try:
            super().start()
            return
        except ApiIdInvalid:
            logger.critical("API_ID and API_HASH combination is incorrect.")
        except AccessTokenInvalid:
            logger.critical("BOT_TOKEN is invalid.")
        except (AuthKeyUnregistered, AuthKeyDuplicated, struct.error):
            logger.critical("Your SESSION is invalid. Please terminate it and generate a new one.")
        except UserDeactivated:
            logger.critical(f"Account deleted. Time for me to rest.")
        except KeyboardInterrupt:
            logger.critical("Keyboard Interrupt. Exiting..")
        logger.info("For support visit @{}".format(self.__support__))
        raise SystemExit

    # def stop(self):
    #     asyncio.get_event_loop().run_until_complete(super().stop())

    @staticmethod
    def list_modules(directory):
        """List all modules in a directory"""
        if not os.path.exists(directory):
            Stark.log(f"No directory named '{directory}' found")
            return
        if "/." not in directory:
            directory = directory.replace('.', '/')
        plugs = []
        for path, _, files in os.walk(directory):
            for name in files:
                if name.endswith(".py"):
                    file = os.path.join(path, name)[:-3].replace("\\", "/").replace("/", ".")
                    plugs.append(file)
        return plugs

    def load_modules(self, plugins: str):
        """Load all modules from a directory or a single module as pyrogram handlers"""
        # if plugins.endswith(".py"):  # Absolute Path
        #     # C Drive can't have relative path to D: Drive in Windows so needs absolute path
        #     plugins, file = plugins.rsplit(self.sep, 1)
        #     modules: list[str] = [file[:-3]]
        # else:
        modules: list[str] = self.list_modules(plugins)
        # plugins = plugins.replace("\\", ".").replace("/", ".")
        if not modules:
            return
        data = {"total_plugins": 0, "all_plugins": {}}
        for module in modules:
            # module = module.replace("\\", "/")
            # module = module.replace("/", ".")
            # if module.endswith(".py"):
            #     module = module[:-3]
            # plugins = plugins.replace("/", ".")
            # # plugins+"."+
            mod = importlib.import_module(module)
            funcs = [func for func, _ in getmembers(mod, isfunction)]
            real_plugs = []
            for func in funcs:
                try:
                    for handler, group in getattr(mod, func).handlers:
                        self.add_handler(handler, group)
                    real_plugs.append(func)
                except AttributeError:
                    pass
                try:
                    method = getattr(mod, func).pytgcalls
                    getattr(self.call, "_on_event_update").add_handler(method, func)
                    real_plugs.append(func)
                except AttributeError:
                    pass
            module = module.replace(".", "/")
            if "plugins" in module:
                x = "plugins"
            else:
                x = "assistant"
            module = module.split(x+"/", 1)[-1]
            if real_plugs:
                data["total_plugins"] += 1
                data["all_plugins"][module] = {"path": mod.__file__, "doc": mod.__doc__}
                # , ", ".join(real_plugs)
                # [{}]
                logger.info("Loaded plugin - {}.py".format(module))
        self.__data__ = data

    @staticmethod
    def log(message, level: str | int = logging.INFO):
        """Log messages to console.

        | String       | Integer  |
        |:------------:|:--------:|
        | **debug**    | 10       |
        | **info**     | 20       |
        | **warning**  | 30       |
        | **error**    | 40       |
        | **critical** | 50       |

        Parameters:

            message (Any): Item to print to console.
            level (optional): Logging level as string or int.
        """
        if isinstance(level, str):
            level = level.lower()
        if level in ["critical", 50]:
            level = logging.CRITICAL
        elif level in ["error", 40]:
            level = logging.ERROR
        elif level in ["warning", "warn", 30]:
            level = logging.WARNING
        elif level in ["info", 20]:
            level = logging.INFO
        elif level in ["debug", 10]:
            level = logging.DEBUG
        logger.log(level, message)

    @property
    def data(self) -> dict:
        dictionary = self.__data__.copy()
        if self.is_bot:
            dictionary.update(bot_command_data)
        else:
            dictionary.update(command_data)
        return dictionary

    @property
    def total_plugins(self) -> int:
        """Number of total plugins loaded."""
        return self.data["total_plugins"]

    @property
    def all_plugins(self) -> dict:  # not "plugins" to prevent clash with pyrogram's "plugins" attribute
        """Dictionary of all plugins and their respective absolute paths.

        Example:

        ```python
        {
            "basic": {
                "path": "C:\\Users\\....\\plugins\\basic.py",
                "doc": "Docstrings"
            },
            "sample": {
                "path": "D:\\Bots\\...\\sample.py",
                "doc": None
            },
        },
        ```
        """
        return self.data["all_plugins"]

    @property
    def total_commands(self) -> int:
        """Number of total commands loaded."""
        return self.data["total_commands"]

    @property
    def all_commands(self) -> dict:
        """Dictionary of all commands and their descriptions if present else None.

        Example:

        ```python
        {
            "start": "Start the bot"
            "help": "Get help"
            "about": "About this bot"
        },
        ```
        """
        return self.data["all_commands"]

    @property
    def sudo_commands(self) -> list[str]:
        """List of all sudo commands available. Includes `owner_only` commands too."""
        return self.data["sudo_commands"]

    # @property
    # def langs(self) -> dict:
    #     """Returns all languages and corresponding file names if localization is set up otherwise None"""
    #     return {f: self.langs_data[f]["language"] for f in self.langs_data}

    @property
    def plugins_cmds(self) -> dict:
        return self.data["plugins_cmds"]

    # async def log_tg(self, text):
    #     """Log a text message to your log chat as defined in environment variable [LOG_CHAT](/start/variables#log_chat)
    #
    #     Parameters:
    #         text: Text that needs to be logged.
    #     """
    #     await self.send_message(env.LOG_CHAT, text)
    #     # No exceptions are handled for now.

    def _set_info(self):
        client = self.get_me()
        self.id = client.id
        self.username = "@"+client.username if client.username else client.first_name
        self.name = client.first_name

    @property
    def help_strings(self):
        return self.data["help_strings"]

    async def load_sudos(self):
        if self.id not in env.OWNER_ID:
            env.OWNER_ID.append(self.id)
        sudos = await mongo.list_sudo()
        listed = env.SUDO_USERS + env.OWNER_ID
        for i in listed:
            if i not in sudos:
                await mongo.add_sudo(i)
        sudos = await mongo.list_sudo()
        for s in sudos:
            if s not in SUDO_USERS:
                SUDO_USERS.append(s)
        return SUDO_USERS.copy()

    async def create_input_document(self, chat_id: int, path: str):
        media = await self.invoke(
            raw.functions.messages.UploadMedia(
                peer=await self.resolve_peer(chat_id),
                media=raw.types.InputMediaUploadedDocument(
                    mime_type=self.guess_mime_type(path) or "application/zip",
                    file=await self.save_file(path),
                    attributes=[
                        raw.types.DocumentAttributeFilename(
                            file_name=os.path.basename(path)
                        )
                    ],
                ),
            )
        )
        input_doc = raw.types.InputDocument(
            id=media.document.id,
            access_hash=media.document.access_hash,
            file_reference=media.document.file_reference,
        )
        return input_doc

    async def add_to_pack(self, chat_id: int, path: str, pack: str, emoji: str):
        i = await self.create_input_document(chat_id, path)
        await self.invoke(
            raw.functions.stickers.AddStickerToSet(
                stickerset=raw.types.InputStickerSetShortName(short_name=pack),
                sticker=raw.types.InputStickerSetItem(document=i, emoji=emoji)
            )
        )

    async def create_pack(
        self,
        chat_id: int,
        user_id: int,
        path: str,
        title: str,
        pack: str,
        emoji: str,
        type: str
    ):
        i = await self.create_input_document(chat_id, path)
        animated = None
        video = None
        if type == "video":
            video = True
        if type == "animated":
            animated = True
        await self.invoke(
            raw.functions.stickers.CreateStickerSet(
                user_id=await self.resolve_peer(user_id),
                title=title,
                short_name=pack,
                stickers=[raw.types.InputStickerSetItem(document=i, emoji=emoji)],
                videos=video,
                animated=animated
            )
        )

    def _buttons_cache(self):
        back_button_text = "<= Back <="
        main: dict = PrettyDict({"back": {}, "plugin": {}, "dirs": {}})
        rows = 2
        plugins_cmds: dict[str, dict[str, str]] = command_data["plugins_cmds"]
        main["back_to_help_button"] = [InlineKeyboardButton(back_button_text, callback_data="help")]
        for directory in plugins_cmds:
            for plugin in plugins_cmds[directory]:
                # b = [InlineKeyboardButton(name[0], callback_data="help+cmd+"+name[0]+"+plugin="+plugin) for name in plugins_cmds[directory][plugin]]
                # bc = [b[i:i + rows] for i in range(0, len(b), rows)]
                back_plug = [InlineKeyboardButton(back_button_text, callback_data="help+"+directory)]
                main["back"][plugin] = InlineKeyboardMarkup([back_plug])
                # bc.append(back_plug)
                # main["plugin"][plugin] = InlineKeyboardMarkup(bc)
                pref = env.CMD_PREFIXES[0]
                li = []
                for name in plugins_cmds[directory][plugin]:
                    x = f"`{pref}{name[0]}`"
                    if len(name) > 1:
                        aliases = [f"`{pref}{f}`" for f in name[1:]]
                        x += f" [alias -  {' | '.join(aliases)}]"
                    li.append(x)
                main["plugin"][plugin] = "**Commands**\n\n"+"\n".join(li)
            b = [InlineKeyboardButton(self.button_title(name), callback_data="help+plugin+"+name) for name in plugins_cmds[directory]]
            bc = [b[i:i + rows] for i in range(0, len(b), rows)]
            bc.append(main["back_to_help_button"])
            main["dirs"][directory] = InlineKeyboardMarkup(bc)
        b = []
        special = None
        for name in plugins_cmds:
            button = InlineKeyboardButton(self.button_title(name), callback_data="help+"+name)
            if name.lower() == "others":
                special = button
            else:
                b.append(button)
        b.append(special)
        bc = [b[i:i + rows] for i in range(0, len(b), rows)]
        main["all"] = InlineKeyboardMarkup(bc)

        # b = [InlineKeyboardButton(self.button_title(name), callback_data="help+plugin+"+name) for name in plugins_cmds]
        # bc = [b[i:i + rows] for i in range(0, len(b), rows)]
        # main["all"] = InlineKeyboardMarkup(bc)
        return main

    @staticmethod
    def button_title(text: str):
        text = text.replace("\\", "/")
        text = text.split("/")[-1].title().replace("_", " ")
        # So id in a word is not replaced
        if text.startswith("id "):
            text = text.replace("id ", "ID ")
        elif text.endswith(" id"):
            text = text.replace(" id", " ID")
        elif " id " in text:
            text = text.replace(" id ", " ID ")
        if text.lower() == "tts":
            text = "TTS"
        if text.lower() == "afk":
            text = "AFK"
        if text.lower() == "vc":
            text = "VC"
        return text

    @staticmethod
    def activate(user: "Stark" = None, bot: "Stark" = None):
        try:
            if not user:
                user = [Stark(s) for s in env.SESSIONS]
            if not bot:
                bot = Stark(is_bot=True)
            user = [user] if not isinstance(user, list) else user
            for i in user:
                call = Calls(i)
                i.call = call
                i.bot = bot
                call.start()
                i.run(start=False)
            bot.run()
            idle()
        finally:
            logger.info("UserBot and Assistant have stopped working. For issues, visit <https://t.me/StarkBotsChat>")


__printed_2__ = False


class Calls(PyTgCalls):
    c: Stark

    def __init__(
        self,
        app: Stark,
        cache_duration: int = 120,
        overload_quiet_mode: bool = False,
        multi_thread: bool = False,
    ):
        app.__class__.__module__ = 'pyrogram.client'
        self.c = app
        global __printed_2__
        if not __printed_2__:
            logger.info("Initializing Laky-64's PyTgCalls <https://github.com/pytgcalls/pytgcalls>")
            __printed_2__ = True
        super().__init__(
            app=app,
            cache_duration=cache_duration,
            overload_quiet_mode=overload_quiet_mode,
            multi_thread=multi_thread
        )

    @staticmethod
    def stream_end():
        def decorator(func):
            setattr(func, "pytgcalls", "STREAM_END_HANDLER")
            return func
        return decorator


PyTgCallsSession.notice_displayed = True



# PyStark2 - Python add-on extension to Pyrogram
# Copyright (C) 2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark2.
#
# PyStark2 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark2 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark2. If not, see <https://www.gnu.org/licenses/>.

from pystark import Stark, Message
from pystark.plugins.helpers import db


@Stark.cmd("broadcast", owner_only=True)
async def broadcast(bot: Stark, msg: Message):
    if not msg.reply_to_message and len(msg.command) == 1:
        await msg.reply("Please reply to a message or pass some text", quote=True)
        return
    text = msg.reply_to_message.text if msg.reply_to_message else msg.text[len("/broadcast "):]
    users = await db.get_all_primary_keys("users")
    await msg.reply_text("Starting Broadcast...")
    count = 0
    for user in users:
        try:
            await bot.send_message(user, text)
            count += 1
        except Exception as e:
            Stark.log(f"Exception occurred while sending message to user with id {user}: {e}", "warning")
    await msg.react(f"Broadcast done successfully to {count} users out of {len(users)}")
